# Temperature Relay Controller

This Python application controls a relay (Yocto-MaxiPowerRelay) based on the room temperature measured by a Yocto-Meteo-V2. The application features a graphical user interface (GUI) created using `tkinter`. It supports hot-plugging and provides real-time updates for temperature and device status.

## Features

- **Real-Time Temperature Monitoring**:
  Displays the current temperature measured by the Yocto-Meteo-V2.

- **Relay Control**:
  Automatically opens or closes the relay based on the temperature:
  - Opens the relay if the temperature exceeds 25°C.
  - Closes the relay if the temperature is 25°C or below.

- **Graphical User Interface**:
  - Displays the temperature, sensor status, relay status, and relay state.
  - Uses `tkinter` for a simple and intuitive interface.

- **Hot-Plugging Support**:
  Automatically detects and reconnects devices if they are unplugged and replugged.

- **Graceful Shutdown**:
  Properly releases resources when the application is closed.

## Prerequisites

1. **Python**:
   - Ensure Python 3.7 or newer is installed.

2. **Yoctopuce Python Library**:
   - Download the library from the [Yoctopuce website](https://www.yoctopuce.com/EN/libraries.php).
   - Extract the library and place it in a known directory.
   - Update the `sys.path.append()` in the script to point to the library's `Sources` folder.

3. **Connected Yoctopuce Modules**:
   - **Yocto-Meteo-V2**: For measuring temperature.
   - **Yocto-MaxiPowerRelay**: For relay control.

## Installation

1. Clone this repository:
   ```bash
   git clone https://github.com/your-username/temperature-relay-controller.git
   cd temperature-relay-controller
   ```

2. Place the Yoctopuce library in the appropriate directory and update the path in the script.

3. Install any missing dependencies:
   ```bash
   pip install tkinter
   ```

## Usage

1. Connect the Yocto-Meteo-V2 and Yocto-MaxiPowerRelay to your computer via USB.
2. Run the application:
   ```bash
   python temperature_relay_gui.py
   ```
3. The GUI will display:
   - The current room temperature.
   - Sensor and relay statuses.
   - Relay state (open/closed).
4. The application will automatically adjust the relay based on the temperature.

## Project Structure

```
temperature-relay-controller/
├── temperature_relay_gui.py  # Main application script
├── README.md                 # Project documentation
└── requirements.txt          # Optional dependencies file
```

## Relay Logic

- **Relay Closed**:
  - Temperature ≤ 25°C.
  - The circuit is completed (normally closed).

- **Relay Open**:
  - Temperature > 25°C.
  - The circuit is disconnected (normally open).

## Hot-Plugging

The application periodically refreshes the device list using `YAPI.UpdateDeviceList()`. If a device is disconnected and reconnected, the application will detect and reinitialize it automatically.

## Troubleshooting

1. **No Devices Detected**:
   - Ensure the devices are connected via USB.
   - Check the path to the Yoctopuce library in the script.

2. **Incorrect Relay Behavior**:
   - Verify that `relay1` is explicitly targeted by the serial number in the script.

3. **Yoctopuce API Errors**:
   - Ensure that the Yoctopuce library is properly extracted and accessible.

## Contributing

Contributions are welcome! Feel free to open issues or submit pull requests.

## License

This project is licensed under the MIT License. See the [LICENSE](LICENSE) file for details.

## Acknowledgments

- [Yoctopuce](https://www.yoctopuce.com/) for providing high-quality USB sensors and relays.
- The Python community for the `tkinter` library.
```

### Additional Notes
- Replace `your-username` in the GitHub clone URL with your GitHub username.
- Add a `requirements.txt` file if additional dependencies are needed. For example:
  ```text
  yoctopuce
  tkinter
  ```
