import sys
import time

# Add the Yoctopuce library path
sys.path.append(r"C:\data\tmp\YoctoLib.python.62875\Sources")

from yocto_api import *
from yocto_temperature import *
from yocto_relay import *

def initialize_modules():
    errmsg = YRefParam()
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        print(f"Initialization error: {errmsg.value}")
        sys.exit(1)

def find_modules():
    temperature_sensor = YTemperature.FirstTemperature()
    relay = YRelay.FirstRelay()

    if not temperature_sensor:
        print("Yocto-Meteo-V2 not found.")
    if not relay:
        print("Yocto-MaxiPowerRelay not found.")

    return temperature_sensor, relay

def manage_modules(temperature_sensor, relay):
    while True:
        temperature_sensor = YTemperature.FindTemperature(temperature_sensor.get_hardwareId()) if temperature_sensor else None
        relay = YRelay.FindRelay(relay.get_hardwareId()) if relay else None

        try:
            if temperature_sensor and temperature_sensor.isOnline():
                temp = temperature_sensor.get_currentValue()
                print(f"Room temperature: {temp}°C")

                if relay and relay.isOnline():
                    if temp > 25:
                        relay.set_state(YRelay.STATE_B)
                        print("Relay opened (temperature > 25°C).")
                    else:
                        relay.set_state(YRelay.STATE_A)
                        print("Relay closed (temperature <= 25°C).")
                else:
                    print("Relay not found.")
            else:
                print("Temperature sensor not found.")

        except YAPI_Exception as e:
            print(f"Error: {e}")

        time.sleep(1)

if __name__ == "__main__":
    initialize_modules()
    temperature_sensor, relay = find_modules()
    
    if not temperature_sensor or not relay:
        print("One or both modules are missing. Ensure they are connected.")
    
    try:
        manage_modules(temperature_sensor, relay)
    except KeyboardInterrupt:
        print("\nExiting program.")
    finally:
        YAPI.FreeAPI()
