import sys
import time

# Add the Yoctopuce library path
sys.path.append(r"C:\data\tmp\YoctoLib.python.62875\Sources")

from yocto_api import *
from yocto_temperature import *
from yocto_relay import *

def initialize_modules():
    errmsg = YRefParam()
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        print(f"Initialization error: {errmsg.value}")
        sys.exit(1)

def find_temperature_sensor():
    # Use the serial number for the Yocto-Meteo-V2
    return YTemperature.FindTemperature("METEOMK2-20713E.temperature")

def find_relay():
    # Use the serial number for relay 1 on the Yocto-MaxiPowerRelay
    return YRelay.FindRelay("MXPWRRLY-45C5B.relay1")

def manage_modules():
    temperature_sensor = None
    relay = None

    while True:
        try:
            # Refresh the list of connected devices
            YAPI.UpdateDeviceList()

            # Re-detect temperature sensor if not found or disconnected
            if not temperature_sensor or not temperature_sensor.isOnline():
                temperature_sensor = find_temperature_sensor()
                if temperature_sensor and temperature_sensor.isOnline():
                    print("Temperature sensor reconnected.")

            # Re-detect relay 1 if not found or disconnected
            if not relay or not relay.isOnline():
                relay = find_relay()
                if relay and relay.isOnline():
                    print("Relay 1 reconnected.")

            # Process temperature and relay if devices are online
            if temperature_sensor and temperature_sensor.isOnline():
                temp = temperature_sensor.get_currentValue()
                print(f"Room temperature: {temp}°C")
                
                if relay and relay.isOnline():
                    if temp > 25:
                        relay.set_state(YRelay.STATE_B)
                        print("Relay opened (temperature > 25°C).")
                    else:
                        relay.set_state(YRelay.STATE_A)
                        print("Relay closed (temperature <= 25°C).")
                else:
                    print("Relay 1 not found or disconnected.")
            else:
                print("Temperature sensor not found or disconnected.")

        except YAPI_Exception as e:
            print(f"Yoctopuce API Error: {e}")

        # Wait before checking again
        time.sleep(1)

if __name__ == "__main__":
    initialize_modules()
    try:
        manage_modules()
    except KeyboardInterrupt:
        print("\nExiting program.")
    finally:
        YAPI.FreeAPI()
