import time
from yocto_api import *
from yocto_temperature import *
from yocto_relay import *

# Initialisation des modules Yoctopuce
def init_modules():
    errmsg = YRefParam()
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        print("Impossible d'initialiser les modules Yoctopuce : ", errmsg.value)
        exit(1)

def find_modules():
    # Recherche du Yocto-Meteo-V2
    temperature_sensor = YTemperature.FirstTemperature()
    if temperature_sensor is None:
        print("Yocto-Meteo-V2 non détecté. Veuillez le brancher.")
        return None, None

    # Recherche du Yocto-MaxiPowerRelay
    relay = YRelay.FirstRelay()
    if relay is None:
        print("Yocto-MaxiPowerRelay non détecté. Veuillez le brancher.")
        return temperature_sensor, None

    return temperature_sensor, relay

def main():
    init_modules()

    print("Recherche des modules Yoctopuce...")
    temperature_sensor, relay = find_modules()

    if temperature_sensor is None or relay is None:
        print("Veuillez vérifier la connexion des modules et redémarrer l'application.")
        exit(1)

    print("Modules détectés avec succès.")
    print("Application en cours d'exécution...")

    while True:
        try:
            # Rafraîchir la liste des modules pour supporter le hot-plug
            YAPI.UpdateDeviceList(YRefParam())
            
            # Vérifier si les modules sont toujours disponibles
            if not temperature_sensor.isOnline() or not relay.isOnline():
                print("Un module a été débranché. Veuillez le reconnecter.")
                temperature_sensor, relay = find_modules()
                if temperature_sensor is None or relay is None:
                    time.sleep(1)
                    continue

            # Lire la température
            temperature = temperature_sensor.get_currentValue()
            print(f"Température actuelle : {temperature:.1f}°C")

            # Changer l'état du relais en fonction de la température
            if temperature > 20.0:
                relay.set_state(YRelay.STATE_B)
                print("Température > 20°C : Relais ouvert.")
            else:
                relay.set_state(YRelay.STATE_A)
                print("Température ≤ 20°C : Relais fermé.")

            time.sleep(1)
        except KeyboardInterrupt:
            print("Arrêt de l'application.")
            break
        except Exception as e:
            print(f"Erreur : {e}")
            time.sleep(1)

    YAPI.FreeAPI()

if __name__ == "__main__":
    main()
