import time
from yocto_api import *
from yocto_temperature import *
from yocto_relay import *


# Initialisation des modules Yoctopuce
def init_modules():
    errmsg = YRefParam()
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        print("Impossible d'initialiser les modules Yoctopuce : ", errmsg.value)
        exit(1)


def find_modules():
    # Recherche du Yocto-Meteo-V2
    temperature_sensor = YTemperature.FirstTemperature()
    if temperature_sensor is None:
        print("Yocto-Meteo-V2 non détecté. Veuillez le brancher.")
    else:
        print("Yocto-Meteo-V2 détecté.")

    # Recherche du relais numéro 1 sur le Yocto-MaxiPowerRelay
    relay = YRelay.FindRelay("RELAY1")
    if not relay.isOnline():
        print("Yocto-MaxiPowerRelay (relais 1) non détecté. Veuillez le brancher.")
        relay = None
    else:
        print("Yocto-MaxiPowerRelay (relais 1) détecté.")

    return temperature_sensor, relay


def main():
    init_modules()

    print("Recherche des modules Yoctopuce...")
    temperature_sensor, relay = find_modules()

    print("Application en cours d'exécution...")
    while True:
        try:
            # Rafraîchir la liste des modules pour supporter le hot-plug
            YAPI.UpdateDeviceList(YRefParam())

            # Vérifier si les modules sont toujours disponibles
            if temperature_sensor is None or not temperature_sensor.isOnline():
                print("Yocto-Meteo-V2 débranché. Tentative de reconnexion...")
                temperature_sensor, _ = find_modules()

            if relay is None or not relay.isOnline():
                print("Yocto-MaxiPowerRelay (relais 1) débranché. Tentative de reconnexion...")
                _, relay = find_modules()

            # Si l'un des modules est toujours manquant, attendre avant de continuer
            if temperature_sensor is None or relay is None:
                time.sleep(1)
                continue

            # Lire la température
            temperature = temperature_sensor.get_currentValue()
            print(f"Température actuelle : {temperature:.1f}°C")

            # Changer l'état du relais en fonction de la température
            if temperature > 25.0:  # Seuil défini à 25°C
                relay.set_state(YRelay.STATE_B)
                print("Température > 25°C : Relais ouvert.")
            else:
                relay.set_state(YRelay.STATE_A)
                print("Température ≤ 25°C : Relais fermé.")

            time.sleep(1)

        except KeyboardInterrupt:
            print("Arrêt de l'application.")
            break
        except Exception as e:
            print(f"Erreur : {e}")
            time.sleep(1)

    YAPI.FreeAPI()


if __name__ == "__main__":
    main()
